# -*- coding: utf-8 -*-
# Copyright (C) 2017 - 2018 Tim Hütz
# This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program. If not,
# see <http://www.gnu.org/licenses/>.

def script_entry_point():
	from argparse import ArgumentParser
	from tomaatti import I3Integration, Tomaatti, ScreenOverlay

	# define the actual argument parser
	argument_parser = ArgumentParser(description='Pomodoro timer for i3')
	argument_parser.add_argument('--screen-overlay', action='store_true', help='Show a full screen overlay (experimental)')
	argument_parser.add_argument('--blur-overlay', action='store_true', help='Blur the background of the screen overlay (experimental)')

	# now we can start parsing the supplied arguments
	parsed_arguments = argument_parser.parse_args()

	# show the experimental screen overlay if the user requested it
	if parsed_arguments.screen_overlay:
		if ScreenOverlay.is_coposite_manager_running():
			print('Showing screen overlay')
			overlay = ScreenOverlay()
			overlay.show_overlay('This is an experimental feature of Tomaatti')
		else:
			print('Composite manager is not running!')
		return

	# get an instance of the main class
	app = Tomaatti()
	app.initialize()

	# toggle the timer if the user performed a right-click
	if I3Integration.get_clicked_button() == I3Integration.RIGHT_MOUSE_BUTTON:
		app.switch_mode()
	elif I3Integration.get_clicked_button() == I3Integration.LEFT_MOUSE_BUTTON:
		app.toggle_timer()

	# check the timer state and react to it
	app.check_state()

	# print the current label to the console
	print(app.current_label)


if __name__ == '__main__':
	script_entry_point()



# -*- coding: utf-8 -*-
# Copyright (C) 2017 - 2018 Tim Hütz
# This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program. If not,
# see <http://www.gnu.org/licenses/>.
from tomaatti import ScreenOverlay
from .configuration import Configuration
from .timertype import TimerType


class Tomaatti(object):
	def initialize(self, configuration_parser=None):
		self._config = Configuration(configuration_parser)

	@staticmethod
	def translate_string(input_text: str) -> str:
		from gettext import gettext, bindtextdomain, textdomain
		bindtextdomain('tomaatti', '/path/to/my/language/directory')  # TODO set a correct path
		textdomain('tomaatti')
		return gettext(input_text)

	@property
	def is_timer_up(self) -> bool:
		from datetime import datetime
		time_to_end = self._config.end_time - datetime.now()
		return time_to_end.days < 0

	@property
	def current_label(self) -> str:
		# determine a prefix for the bar
		prefix = ''
		if self._config.use_fontawesome:
			prefix = '&#xf017;  '

		# determine the actual label for the bar
		if not self._config.is_running:
			return '{}{}'.format(prefix, self.translate_string('Tomaatti'))
		else:
			from datetime import datetime
			time_to_end = self._config.end_time - datetime.now()
			return '{}{:02}:{:02}'.format(prefix, time_to_end.seconds % 3600 // 60, time_to_end.seconds % 60)

	def toggle_timer(self) -> None:
		from datetime import datetime, timedelta

		# toggle the current state
		self._config.is_running = not self._config.is_running

		# if we started the timer, set the target time accordingly
		if self._config.is_running:
			current_time = datetime.now()
			period = self._config.work_duration
			if TimerType.BREAK == self._config.mode:
				period = self._config.break_duration
			time_period = timedelta(minutes=period)
			end_time = current_time + time_period
			self._config.end_time = end_time

	def show_message(self, message: str) -> None:
		if not self._config.use_overlay:
			from easygui import msgbox
			msgbox(message, Tomaatti.translate_string('Tomaatti'))
		else:
			overlay = ScreenOverlay()
			overlay.show_overlay(message)

	def check_state(self):
		if self._config.is_running and self.is_timer_up:
			self.toggle_timer()
			if TimerType.WORKING == self._config.mode:
				self.show_message(self.translate_string(
					"It's time for a break. You worked so hard for the last %d minutes :)" % self._config.work_duration))
				self._config.mode = TimerType.BREAK
			elif TimerType.BREAK == self._config.mode:
				self.show_message(self.translate_string(
					"You had %d minutes of break. Time to start working again!" % self._config.break_duration))
				self._config.mode = TimerType.WORKING
			else:
				self.show_message(self.translate_string('ERROR: %s') % str(self._config.mode))
			self.toggle_timer()

	def switch_mode(self):
		was_running = False
		if self._config.is_running:
			self.toggle_timer()
			was_running = True
		if TimerType.WORKING == self._config.mode:
			self._config.mode = TimerType.BREAK
		else:
			self._config.mode = TimerType.WORKING
		if was_running:
			self.toggle_timer()


# -*- coding: utf-8 -*-
# Copyright (C) 2017 - 2018 Tim Hütz
# This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program. If not,
# see <http://www.gnu.org/licenses/>.


class ScreenOverlay(object):
	@staticmethod
	def is_coposite_manager_running():
		from subprocess import Popen, PIPE
		child = Popen(['pgrep', 'xcompmgr'], stdout=PIPE)
		child.communicate()
		if 0 == child.returncode:
			return True
		else:
			return False

	def __init__(self):
		self._tkinter_root = None
		self._overlay_canvas = None

	def show_overlay(self, text_to_show):
		from tkinter import Canvas, Tk

		self._tkinter_root = Tk()

		self._screen_width = self._tkinter_root.winfo_screenwidth()
		self._screen_height = self._tkinter_root.winfo_screenheight()

		self._overlay_canvas = Canvas(width=self._screen_width, height=self._screen_height, highlightthickness=0)
		self._overlay_canvas.configure(background='black')
		self._overlay_canvas.master.overrideredirect(True)
		self._overlay_canvas.master.geometry('+0+0')
		self._overlay_canvas.master.lift()
		self._overlay_canvas.master.wm_attributes('-topmost', True)
		self._overlay_canvas.master.wm_attributes('-fullscreen', True)
		self._overlay_canvas.master.wm_attributes('-zoomed', False)
		self._overlay_canvas.master.wm_attributes('-alpha', 0.7)
		self._overlay_canvas.create_rectangle(0, 0, self._screen_width, self._screen_height, fill='black')
		self._overlay_canvas.bind('<Button-1>', self._close_callback)
		self._overlay_canvas.pack()

		self._text = self._overlay_canvas.create_text(0, 0, fill="white", font="Roboto 30 bold", text=text_to_show)
		self._overlay_canvas.move(self._text, self._screen_width / 2, self._screen_height / 2)

		self._overlay_canvas.mainloop()

	def _close_callback(self, event):
		self._tkinter_root.destroy()



# -*- coding: utf-8 -*-
# Copyright (C) 2017 - 2018 Tim Hütz
# This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program. If not,
# see <http://www.gnu.org/licenses/>.

class ConfigHelper(object):
	@staticmethod
	def bool_to_config_str(input_val: bool) -> str:
		if input_val:
			return 'yes'
		return 'no'



# -*- coding: utf-8 -*-
# Copyright (C) 2017 - 2018 Tim Hütz
# This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program. If not,
# see <http://www.gnu.org/licenses/>.

from tomaatti.internal.configuration import Configuration
from unittest import TestCase


class ConfigurationTest(TestCase):
	def testTodo(self):
		foo = Configuration()
		self.assertTrue(True)


# -*- coding: utf-8 -*-
# Copyright (C) 2017 - 2018 Tim Hütz
# This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program. If not,
# see <http://www.gnu.org/licenses/>.

class I3Integration(object):
	CLICKED_BUTTON_ENVIRON = 'BLOCK_BUTTON'
	BLOCK_NAME_ENVIRON = 'BLOCK_NAME'
	BLOCK_INSTANCE_ENVIRON = 'BLOCK_INSTANCE'
	LEFT_MOUSE_BUTTON = 1
	MIDDLE_MOUSE_BUTTON = 2
	RIGHT_MOUSE_BUTTON = 3
	MOUSE_SCROLL_UP = 4
	MOUSE_SCROLL_DOWN = 5

	@staticmethod
	def get_clicked_button() -> int:
		from os import environ
		if I3Integration.CLICKED_BUTTON_ENVIRON in environ:
			try:
				return int(environ[I3Integration.CLICKED_BUTTON_ENVIRON])
			except ValueError:
				return -1
		return -1

	@staticmethod
	def get_block_name() -> str:
		from os import environ
		if I3Integration.BLOCK_NAME_ENVIRON in environ:
			return environ[I3Integration.BLOCK_NAME_ENVIRON]
		return ''

	@staticmethod
	def get_block_instance() -> str:
		from os import environ
		if I3Integration.BLOCK_INSTANCE_ENVIRON in environ:
			return environ[I3Integration.BLOCK_INSTANCE_ENVIRON]
		return ''



# -*- coding: utf-8 -*-
# Copyright (C) 2017 - 2018 Tim Hütz
# This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program. If not,
# see <http://www.gnu.org/licenses/>.

from datetime import datetime

from tomaatti import TimerType, ConfigHelper


class Configuration(object):
	def __init__(self, configuration_parser=None):
		from os.path import expanduser, exists, join
		from os import makedirs
		from configparser import ConfigParser

		# ensure the root directory for the configuration files exist
		self._config_directory = expanduser('~/.config/tomaatti')
		if not exists(self._config_directory):
			makedirs(self._config_directory)

		# determine the name of some essential configuration files
		self._config_app_state = join(self._config_directory, 'application_state.ini')

		# if a configuration file exists, read it or use the injected configuration parser
		if configuration_parser:
			self._application_config = configuration_parser
		else:
			self._application_config = ConfigParser()
			self._create_initial_config()
		if exists(self._config_app_state):
			self._application_config.read(self._config_app_state)

	def _create_initial_config(self) -> None:
		# create the sections for the confiugration
		self._application_config.add_section('timer')
		self._application_config.add_section('periods')
		self._application_config.add_section('experimental')
		self._application_config.add_section('ui')

		# assign the default values to the configuration sections
		self._application_config.set('timer', 'mode', str(TimerType.WORKING.value))
		self._application_config.set('timer', 'is_running', ConfigHelper.bool_to_config_str(False))
		self._application_config.set('timer', 'end_time', datetime.now().strftime('%Y-%m-%d %H:%M:%S'))

		self._application_config.set('periods', 'working', '25')
		self._application_config.set('periods', 'break', '5')

		self._application_config.set('ui', 'fontawesome', ConfigHelper.bool_to_config_str(False))

		self._application_config.set('experimental', 'overlay', ConfigHelper.bool_to_config_str(False))
		self._application_config.set('experimental', 'blur', ConfigHelper.bool_to_config_str(False))

	@property
	def mode(self) -> TimerType:
		return TimerType(self._application_config.getint('timer', 'mode'))

	@mode.setter
	def mode(self, value: TimerType) -> None:
		self._application_config.set('timer', 'mode', str(value.value))
		self._persist_current_state()

	@property
	def is_running(self) -> bool:
		return self._application_config.getboolean('timer', 'is_running')

	@is_running.setter
	def is_running(self, value: bool) -> None:
		self._application_config.set('timer', 'is_running', ConfigHelper.bool_to_config_str(value))
		self._persist_current_state()

	@property
	def end_time(self) -> datetime:
		return datetime.strptime(self._application_config.get('timer', 'end_time'), '%Y-%m-%d %H:%M:%S')

	@end_time.setter
	def end_time(self, value: datetime) -> None:
		self._application_config.set('timer', 'end_time', value.strftime('%Y-%m-%d %H:%M:%S'))
		self._persist_current_state()

	@property
	def work_duration(self) -> int:
		return self._application_config.getint('periods', 'working')

	@property
	def break_duration(self) -> int:
		return self._application_config.getint('periods', 'break')

	@property
	def use_fontawesome(self) -> bool:
		return self._application_config.getboolean('ui', 'fontawesome')

	@property
	def use_overlay(self) -> bool:
		return self._application_config.getboolean('experimental', 'overlay')

	@property
	def use_blur(self) -> bool:
		return self._application_config.getboolean('experimental', 'blur')

	def _persist_current_state(self) -> None:
		with open(self._config_app_state, 'w') as configfile:
			self._application_config.write(configfile)
